@@ -1,6 +1,4 @@
 <?php
-/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */
-
 /**
  * Call graph generation class.
  *
@@ -38,7 +36,7 @@ require_once 'drivers/TextDriver.php';
  */
 class PHPCallGraph {
 
-    const VERSION = '0.6.3';
+    const VERSION = '0.6.4';
 
     protected $internalFunctions;
     protected $internalKeywords;
@@ -125,9 +123,29 @@ class PHPCallGraph {
     }
 
     public function analyseCodeSummary() {
-        //TODO: procedural code
+        $this->buildLookupTables();
+
+        //TODO: analyze code in the global scope
+        // currently a workarround is to manually wrap such code
+        // in a dummy function called dummyFunctionForFile_filename_php()
+
+        // analyze functions
+        if (!empty($this->codeSummary['functions'])) {
+            foreach ($this->codeSummary['functions'] as $functionName => $function) {
+                $this->parseMethodBody(
+                        '-',
+                        $functionName,
+                        array(),
+                        array(),
+                        $function['file'],
+                        $function['startLine'],
+                        $function['endLine']
+                        );
+            }
+        }
+
+        // analyze classes
         if (!empty($this->codeSummary['classes'])) {
-            $this->buildLookupTables();
             foreach ($this->codeSummary['classes'] as $className => $class) {
                 //echo $className, "\n";
                 if (!empty($class['methods'])) {
@@ -172,16 +190,34 @@ class PHPCallGraph {
         }
     }
 
+    /**
+     * @param string  $className
+     * @param string  $methodName
+     * @param array   $propertyNames
+     * @param array   $methodNames
+     * @param string  $file
+     * @param integer $startLine
+     * @param integer $endLine
+     */
     public function parseMethodBody(
         $className,
         $methodName,
-        array $propertyNames,
-        array $methodNames,
+        Array $propertyNames,
+        Array $methodNames,
         $file,
         $startLine,
         $endLine
     ) {
-        $callerName = $className . '::' . $methodName . $this->generateParametersForSignature($this->codeSummary['classes'][$className]['methods'][$methodName]['params']);
+        if ($className == '-') { // we are analyzing a function not a method
+            if (substr($methodName, 0, strlen('dummyFunctionForFile_')) == 'dummyFunctionForFile_') {
+                // the function has been introduced manually to encapsulate code in the global scope
+                $callerName = str_replace('_', '.', substr($methodName, strlen('dummyFunctionForFile_'))) . '(File)';
+            } else {
+                $callerName = $methodName . $this->generateParametersForSignature($this->codeSummary['functions'][$methodName]['params']);
+            }
+        } else {
+            $callerName = $className . '::' . $methodName . $this->generateParametersForSignature($this->codeSummary['classes'][$className]['methods'][$methodName]['params']);
+        }
 
         $offset = $startLine - 1;
         $length = $endLine - $startLine + 1;
@@ -205,10 +241,13 @@ class PHPCallGraph {
         }
         //*/
 
+        //TODO: implement a higher level API for working with PHP parser tokens (e.g. TokenIterator)
         foreach ($tokens as $i => $token) {
             //TODO: obtain method signature directly from the source file
-            $lineNumber+= substr_count($token[1], "\n");
-            
+            if (is_array($token)) {
+                $lineNumber+= substr_count($token[1], "\n");
+            }
+
             /*
             if (count($token) == 3) {
                 echo "\t", token_name($token[0]), "\n";
@@ -298,13 +337,13 @@ class PHPCallGraph {
                             }
                         } elseif (
                             (
-                                $peviousPreviousToken[1] == '$this'
+                                isset($peviousPreviousToken[1]) and $peviousPreviousToken[1] == '$this'
                                 and $peviousToken[0]     == T_OBJECT_OPERATOR
                                 and in_array($token[1], $methodNames)
                             )
                             or
                             (
-                                $peviousPreviousToken[1] == 'self'
+                                isset($peviousPreviousToken[1]) and $peviousPreviousToken[1] == 'self'
                                 and $peviousToken[0]     == T_DOUBLE_COLON
                                 and in_array($token[1], $methodNames)
                             )
